@@ -1,7 +1,7 @@
 import re
-import os
 import sys
 import json
+import pathlib
 import logging
 import traceback
 
@@ -49,7 +49,7 @@ def format_scrub_warning(warning):
     """
 
     # Format the warning
-    scrub_warning = (warning.get('id') + ' <' + warning.get('priority') + '> :' + warning.get('file') + ':' +
+    scrub_warning = (warning.get('id') + ' <' + warning.get('priority') + '> :' + str(warning.get('file')) + ':' +
                      str(warning.get('line')) + ': ' + warning.get('query') + '\n')
 
     # Add the description
@@ -82,7 +82,7 @@ def create_scrub_output_file(warnings, output_file):
                 output_fh.write(scrub_warning)
 
     # Change the permissions of the output file
-    os.chmod(output_file, 438)
+    output_file.chmod(0o644)
 
 
 def verify_sarif(sarif_data):
@@ -228,18 +228,17 @@ def parse_scrub(scrub_file, source_root):
 
         # Get the values of interest
         warning_id = warning_info[0].split()[0]
-        warning_file = warning_info[1]
+        warning_file = pathlib.Path(warning_info[1])
         warning_line = int(warning_info[2])
         warning_tool = re.sub(r'[0-9]', '', warning_id)
         warning_priority = re.sub('[<>]', '', warning_info[0].split()[-1])
 
         # Update the file path, if necessary
-        if not warning_file.startswith('/'):
-            warning_file = os.path.join(source_root, warning_file)
-        warning_file = os.path.realpath(warning_file)
+        if warning_file.anchor != '/':
+            warning_file = source_root.joinpath(warning_file).resolve()
 
         # Add the warning to the dictionary
-        warning_list.append(create_warning(warning_id, warning_file, warning_line, warning_description, warning_tool,
+        warning_list.append(create_warning(warning_id, warning_file.resolve(), warning_line, warning_description, warning_tool,
                                            warning_priority, warning_query))
 
     return warning_list
@@ -284,7 +283,7 @@ def parse_sarif(sarif_filename, source_root, id_prefix=None):
             # Update the source root if it can be found in the SARIF data
             if "originalUriBaseIds" in sarif_data:
                 for item in sarif_data.get("originalUriBaseIds").items():
-                    updated_source_dir = os.path.realpath(item[-1]['uri'].replace('file://', ''))
+                    updated_source_dir = pathlib.Path(item[-1]['uri'].replace('file://', '')).resolve()
 
             # TODO: replace key checks with booleans for all optional SARIF fields
             tool_name, analysis_rules, locations = '', [], []
@@ -357,17 +356,16 @@ def parse_sarif(sarif_filename, source_root, id_prefix=None):
                         warning_line = 0
 
                     # Fix the filepath
-                    warning_file = warning_file.replace('file://', '')
-                    if not warning_file.startswith('/'):
-                        warning_file = os.path.join(updated_source_dir, warning_file)
-                    warning_file = os.path.realpath(warning_file)
+                    warning_file = pathlib.Path(warning_file.replace('file://', ''))
+                    if warning_file.anchor != '/':
+                        warning_file = updated_source_dir.joinpath(warning_file)
 
                     # Set the warning ID
                     warning_id = tool_name + str(warning_count).zfill(3)
 
                     # Add to the warning dictionary
-                    results.append(create_warning(warning_id, warning_file, warning_line, warning_description, tool_name,
-                                                  'Low', warning_query, suppress_warning))
+                    results.append(create_warning(warning_id, warning_file.resolve(), warning_line, warning_description,
+                                                  tool_name, 'Low', warning_query, suppress_warning))
 
                     # Update the warning count
                     warning_count = warning_count + 1
@@ -416,7 +414,6 @@ def create_sarif_output_file(results_list, sarif_version, output_file, source_ro
         file_index = 0
 
         # Set the priority level
-        #result_item['level'] = warning['priority']
         result_item['level'] = 'warning'
 
         # Set the rule ID
@@ -428,9 +425,9 @@ def create_sarif_output_file(results_list, sarif_version, output_file, source_ro
             }
         if sarif_version == '2.0.0':
             # TODO: CAREFUL USING 2.0.0, STRUCT IS NOT COMPLETELY DEPENDABLE, NEEDS WORK
-            sarif_output['runs'][0]['resources'] = {'rules': rules_list}
+            sarif_output['runs'][0].update({'resources': rules_list})
 
-            sarif_output['runs'][0]['tool'] = results_list[0]['toolName']
+            sarif_output['runs'][0].update({'tool': results_list[0]['tool']})
             # TODO: get the logic right for deciding mime type... pass file_list like rules_list
             sarif_output['runs'][0]['files'] = [{
                 "mimeType": "text/c",
@@ -472,8 +469,8 @@ def create_sarif_output_file(results_list, sarif_version, output_file, source_ro
             result_item['locations'] = [{
                 'physicalLocation': {
                     'artifactLocation': {
-                        'uri': os.path.relpath(warning['file'], source_root),
-                        'uriBaseId': source_root
+                        'uri': str(warning['file'].relative_to(source_root)),
+                        'uriBaseId': str(source_root)
                     },
                     'region': {
                         'startLine': warning['line']
@@ -491,7 +488,7 @@ def create_sarif_output_file(results_list, sarif_version, output_file, source_ro
         json.dump(sarif_output, output_fh, indent=4)
 
     # Change the permissions of the output file
-    os.chmod(output_file, 438)
+    output_file.chmod(0o644)
 
 
 def perform_translation(input_file, output_file, source_root, output_format):
@@ -513,10 +510,10 @@ def perform_translation(input_file, output_file, source_root, output_format):
 
     try:
         # Parse the input file
-        if input_file.endswith('.scrub'):
+        if input_file.suffix == '.scrub':
             parsed_results = parse_scrub(input_file, source_root)
 
-        elif input_file.endswith('.sarif'):
+        elif input_file.suffix == '.sarif':
             parsed_results = parse_sarif(input_file, source_root)
 
         else:
@@ -553,4 +550,4 @@ def perform_translation(input_file, output_file, source_root, output_format):
 
 
 if __name__ == '__main__':
-    perform_translation(sys.argv[1], sys.argv[2], sys.argv[3], sys.argv[4])
+    perform_translation(pathlib.Path(sys.argv[1]), pathlib.Path(sys.argv[2]), pathlib.Path(sys.argv[3]), sys.argv[4])
