@@ -1,6 +1,4 @@
-import re
 import os
-import glob
 import logging
 import traceback
 from scrub.utils.filtering import create_file_list
@@ -18,7 +16,7 @@ def initialize_analysis(scrub_conf_data):
     """
 
     scrub_conf_data.update({'filter_warnings': True})
-    scrub_conf_data.update({'filtering_log_file': scrub_conf_data.get('scrub_log_dir') + '/filtering.log'})
+    scrub_conf_data.update({'filtering_log_file': scrub_conf_data.get('scrub_log_dir').joinpath('filtering.log')})
 
     return scrub_conf_data
 
@@ -36,16 +34,16 @@ def filter_scrub_results(scrub_conf_data):
                                       scrub_conf_data.get('analysis_filters'))
 
     # Get the list of SCRUB files
-    results_files = glob.glob(scrub_conf_data.get('raw_results_dir') + '/*.scrub')
+    results_files = scrub_conf_data.get('raw_results_dir').glob('*.scrub')
 
     # Sort the files into groups
     raw_compiler_files = []
     raw_p10_files = []
     raw_generic_files = []
     for results_file in results_files:
-        if re.search(r'compiler_raw\.scrub', results_file):
+        if 'compiler_raw' in results_file.stem:
             raw_compiler_files.append(results_file)
-        elif re.search(r'p10_raw\.scrub', results_file):
+        elif 'p10_raw' in results_file.stem:
             raw_p10_files.append(results_file)
         else:
             raw_generic_files.append(results_file)
@@ -54,19 +52,18 @@ def filter_scrub_results(scrub_conf_data):
     if raw_compiler_files:
         try:
             # Set the compiler output file path
-            filtered_compiler_results_file = scrub_conf_data.get('scrub_analysis_dir') + '/compiler.scrub'
+            filtered_compiler_results_file = scrub_conf_data.get('scrub_analysis_dir').joinpath('compiler.scrub')
 
             # Parse all of the input files
             compiler_results = []
             valid_warning_types = []
             for results_file in raw_compiler_files:
                 # Append the results file
-                compiler_results = (compiler_results +
-                                    translate_results.parse_scrub(results_file,
-                                                                  scrub_conf_data.get('source_dir')))
+                compiler_results = (compiler_results + translate_results.parse_scrub(results_file,
+                                                                                     scrub_conf_data.get('source_dir')))
 
                 # Append to the valid warning types
-                valid_warning_types.append(os.path.basename(results_file).split('_')[0])
+                valid_warning_types.append(results_file.stem.split('_')[0])
 
             # Filter the results file
             filter_results.filter_results(compiler_results, filtered_compiler_results_file,
@@ -88,7 +85,7 @@ def filter_scrub_results(scrub_conf_data):
     if raw_p10_files:
         try:
             # Set the output file path
-            filtered_p10_results = scrub_conf_data.get('scrub_analysis_dir') + '/p10.scrub'
+            filtered_p10_results = scrub_conf_data.get('scrub_analysis_dir').joinpath('p10.scrub')
 
             # Parse all of the input files
             p10_results = []
@@ -99,7 +96,7 @@ def filter_scrub_results(scrub_conf_data):
                                                                            scrub_conf_data.get('source_dir')))
 
                 # Append to the valid warning types
-                valid_warning_types.append(os.path.basename(results_file).split('_')[0])
+                valid_warning_types.append(results_file.stem.split('_')[0])
 
             filter_results.filter_results(p10_results, filtered_p10_results,
                                           scrub_conf_data.get('filtering_output_file'),
@@ -125,8 +122,8 @@ def filter_scrub_results(scrub_conf_data):
                                                                      scrub_conf_data.get('source_dir'))
 
                 # Get the output file name
-                tool_name = os.path.basename(raw_generic_file).split('_')[0]
-                filtered_generic_results = scrub_conf_data.get('scrub_analysis_dir') + '/' + tool_name + '.scrub'
+                tool_name = raw_generic_file.stem.split('_')[0]
+                filtered_generic_results = scrub_conf_data.get('scrub_analysis_dir').joinpath(tool_name + '.scrub')
 
                 filter_results.filter_results(raw_generic_warnings, filtered_generic_results,
                                               scrub_conf_data.get('filtering_output_file'),
@@ -134,7 +131,7 @@ def filter_scrub_results(scrub_conf_data):
                                               scrub_conf_data.get('source_dir'),
                                               scrub_conf_data.get('enable_micro_filter'),
                                               scrub_conf_data.get('enable_ext_warnings'),
-                                              tool_name)
+                                              [tool_name])
 
             except:     # lgtm [py/catch-base-exception]
                 # Print a status message
@@ -145,7 +142,7 @@ def filter_scrub_results(scrub_conf_data):
 
     # Execute the custom filtering command if it exists
     if scrub_conf_data.get('custom_filter_cmd'):
-        scrub_utilities.execute_command(scrub_conf_data.get('custom_filter_cmd'), os.environ.copy())
+        scrub_utilities.execute_command(str(scrub_conf_data.get('custom_filter_cmd')), os.environ.copy())
 
 
 def generate_sarif(scrub_conf_data):
@@ -156,23 +153,23 @@ def generate_sarif(scrub_conf_data):
     """
 
     # Find all of the SCRUB output files
-    scrub_files = glob.glob(scrub_conf_data.get('scrub_analysis_dir') + '/*.scrub')
+    scrub_files = scrub_conf_data.get('scrub_analysis_dir').glob('*.scrub')
 
     for scrub_file in scrub_files:
         # Create the SARIF output file path
-        sarif_output_file = (scrub_conf_data.get('sarif_results_dir') + '/' +
-                             os.path.splitext(os.path.basename(scrub_file))[0] + '.sarif')
+        sarif_output_file = scrub_conf_data.get('sarif_results_dir').joinpath(scrub_file.stem + '.sarif')
 
         # Create a SARIF output file
         translate_results.perform_translation(scrub_file, sarif_output_file, scrub_conf_data.get('source_dir'),
                                               'sarifv2.1.0')
 
 
-def run_analysis(scrub_conf_data, override=False):
+def run_analysis(scrub_conf_data, console_logging=logging.INFO, override=False):
     """This function performs results filtering of raw analysis results.
 
     Inputs:
         - baseline_conf_data: Dictionary of raw scrub.cfg configuration parameters [dict]
+        - console_logging: Level of console logging information to print to console [optional] [enum]
         - override: Force tool execution? [optional] [bool]
 
     Outputs:
@@ -191,7 +188,7 @@ def run_analysis(scrub_conf_data, override=False):
     if attempt_analysis:
         try:
             # Create the logger
-            scrub_utilities.create_logger(filtering_conf_data.get('filtering_log_file'))
+            scrub_utilities.create_logger(filtering_conf_data.get('filtering_log_file'), console_logging)
 
             # Print a status message
             logging.info('')
@@ -204,14 +201,14 @@ def run_analysis(scrub_conf_data, override=False):
             filter_scrub_results(scrub_conf_data)
 
             # Check the status of all the filtered SCRUB output files
-            for output_file in glob.glob(os.path.join(scrub_conf_data['scrub_analysis_dir'], '*.scrub')):
+            for output_file in scrub_conf_data.get('scrub_analysis_dir').glob('*.scrub'):
                 scrub_utilities.check_artifact(output_file, False)
 
             # Convert the results into SARIF format
             generate_sarif(scrub_conf_data)
 
             # Check the status of all the filtered SARIF output files
-            for output_file in glob.glob(os.path.join(scrub_conf_data['scrub_analysis_dir'], 'sarif_results/*.sarif')):
+            for output_file in scrub_conf_data.get('scrub_analysis_dir').glob('sarif_results/*.sarif'):
                 scrub_utilities.check_artifact(output_file, False)
 
             # Set the exit code
@@ -221,7 +218,7 @@ def run_analysis(scrub_conf_data, override=False):
             # Print a warning message
             logging.warning('Filtering and distribution could not be completed. '
                             'Please see log file %s for more information.',
-                            filtering_conf_data.get('filtering_log_file'))
+                            str(filtering_conf_data.get('filtering_log_file')))
 
             # Print the exception traceback
             logging.warning(traceback.format_exc())
